# pylint: disable=wildcard-import, unused-wildcard-import, unused-import
# flake8: noqa: F405
from .admin_action import admin_action
from .category_table import CategoryTable
from .fields import *
from .fields_schema import FieldsSchema
from .table_models import (
    AutocompleteData, AutocompleteResult, CreateResult, ListData, RetrieveResult, TableListResult, UpdateResult)



import pytest
from fastapi.testclient import TestClient

from brilliance_admin.schema.table.admin_action import ActionData
from example.main import app
from example.sections.models import CurrencyFactory, Merchant, MerchantFactory, TerminalFactory

client = TestClient(app)


@pytest.mark.asyncio
async def test_exception_handle(mocker):
    url = app.url_path_for(
        'table_action',
        group='payments',
        category='payments',
        action='action_with_exception',
    )
    request_data = ActionData()
    response = client.post(url, json=request_data.model_dump(mode='json'))
    assert response.status_code == 500, response.content.decode()
    response_data = {
        'code': 'user_action_error',
        'field_errors': None,
        'message': 'Exception example.',
    }
    assert response.json() == response_data


@pytest.mark.asyncio
async def test_change_password(mocker):
    url = app.url_path_for(
        'table_action',
        group='users',
        category='user',
        action='change_password',
    )
    request_data = ActionData(
        pks=[70],
        form_data={"new_password": "123123"},
    )
    response = client.post(url, json=request_data.model_dump(mode='json'))
    assert response.status_code == 200, response.content.decode()
    assert response.json() == {"message": "Password successfully updated", "persistent_message": None}


@pytest.mark.asyncio
async def test_delete_payment(mocker):
    url = app.url_path_for(
        'table_action',
        group='payments',
        category='payments',
        action='delete',
    )
    request_data = ActionData(
        pks=[100500],
    )
    response = client.post(url, json=request_data.model_dump(mode='json'))
    assert response.status_code == 200, response.content.decode()
    assert response.json() == {"message": "The entries were successfully deleted.", "persistent_message": None}


@pytest.mark.asyncio
async def test_delete(postgres_sessionmaker):
    merchant = await MerchantFactory()
    url = app.url_path_for(
        'table_action',
        group='payments',
        category='merchant',
        action='delete',
    )
    request_data = ActionData(
        pks=[merchant.id],
    )
    response = client.post(url, json=request_data.model_dump(mode='json'))
    assert response.status_code == 200, response.content.decode()
    assert response.json() == {"message": "The entries were successfully deleted.", "persistent_message": None}

    async with postgres_sessionmaker() as session:
        result = await session.get(Merchant, merchant.id)
        assert result is None


@pytest.mark.asyncio
async def test_delete_integrity_error(postgres_sessionmaker):
    merchant = await MerchantFactory()
    currency = await CurrencyFactory()
    await TerminalFactory(merchant=merchant, currency=currency)

    url = app.url_path_for(
        'table_action',
        group='payments',
        category='merchant',
        action='delete',
    )
    request_data = ActionData(
        pks=[merchant.id],
    )
    response = client.post(url, json=request_data.model_dump(mode='json'))
    assert response.status_code == 400, response.content.decode()
    assert response.json() == {
        "message": "Cannot delete: related records exist: terminal.merchant_id",
        "code": None,
        "field_errors": None,
    }

    async with postgres_sessionmaker() as session:
        result = await session.get(Merchant, merchant.id)
        assert result is not None



from unittest import mock

import pytest

from brilliance_admin import schema, sqlalchemy
from brilliance_admin.auth import UserABC
from example.sections.models import Terminal

category_schema_data = {
    'dashboard_info': None,
    'icon': None,
    'description': None,
    'link': None,
    'categories': {},
    'table_info': {
        'actions': {
            'delete': {
                'allow_empty_selection': False,
                'base_color': 'red-lighten-2',
                'confirmation_text': 'Вы уверены, что хотите удалить данные записи?\n'
                'Данное действие нельзя отменить.',
                'description': None,
                'form_schema': None,
                'icon': None,
                'title': 'Удалить',
                'variant': 'outlined',
            },
        },
        'can_create': True,
        'can_retrieve': True,
        'can_update': True,
        'ordering_fields': [],
        'default_ordering': '-id',
        'pk_name': 'id',
        'search_enabled': True,
        'search_help': mock.ANY,
        'table_filters': None,
        'table_schema': {
            'fields': {
                'created_at': {
                    'header': {},
                    'label': 'Created At',
                    'read_only': False,
                    'required': False,
                    'type': 'datetime',
                    'range': True,
                    'include_date': True,
                    'include_time': True,
                },
                'currency_id': {
                    'dual_list': False,
                    'header': {},
                    'label': 'Currency',
                    'many': False,
                    'read_only': False,
                    'rel_name': 'currency',
                    'required': True,
                    'type': 'related',
                },
                'description': {
                    'header': {},
                    'label': 'Описание',
                    'max_length': 255,
                    'password': False,
                    'read_only': False,
                    'required': True,
                    'type': 'string',
                },
                'id': {
                    'header': {},
                    'label': 'ID',
                    'read_only': True,
                    'required': False,
                    'type': 'integer',
                },
                'is_h2h': {
                    'header': {},
                    'label': 'Is H2H',
                    'read_only': False,
                    'required': False,
                    'type': 'boolean',
                },
                'merchant_id': {
                    'dual_list': False,
                    'header': {},
                    'label': 'Merchant',
                    'many': False,
                    'read_only': False,
                    'rel_name': 'merchant',
                    'required': True,
                    'type': 'related',
                },
                'registered_delay': {
                    'header': {},
                    'label': 'Registered Delay',
                    'read_only': False,
                    'required': False,
                    'type': 'integer',
                },
                'secret_key': {
                    'header': {},
                    'label': 'Secret Key',
                    'max_length': 255,
                    'password': False,
                    'read_only': False,
                    'required': False,
                    'type': 'string',
                },
                'title': {
                    'header': {},
                    'label': 'Title',
                    'max_length': 255,
                    'password': False,
                    'read_only': False,
                    'required': True,
                    'type': 'string',
                },
                'status': {
                    'choices': [
                        {
                            'tag_color': 'grey-lighten-1',
                            'title': 'В процессе',
                            'value': 'process',
                        },
                        {
                            'tag_color': 'green-darken-1',
                            'title': 'Успех',
                            'value': 'success',
                        },
                        {
                            'tag_color': 'red-lighten-2',
                            'title': 'Ошибка',
                            'value': 'error',
                        },
                    ],
                    'header': {},
                    'label': 'Status',
                    'read_only': False,
                    'required': True,
                    'size': 'default',
                    'type': 'choice',
                    'variant': 'elevated',
                },
                'fee_id': {
                    'dual_list': False,
                    'header': {},
                    'label': 'Fee',
                    'many': False,
                    'read_only': False,
                    'rel_name': 'fee',
                    'required': False,
                    'type': 'related',
                },
                'manager_id': {
                    'header': {},
                    'label': 'Manager ID',
                    'read_only': False,
                    'required': True,
                    'type': 'integer',
                },
            },
            'list_display': [
                'id',
                'manager_id',
                'title',
                'fee_id',
                'status',
                'description',
                'secret_key',
                'currency_id',
                'merchant_id',
                'is_h2h',
                'registered_delay',
                'created_at',
            ],
        },
    },
    'title': 'Terminal',
    'type': 'table',
}

FIELDS = [
    'id',
    'manager_id',
    'title',
    'fee_id',
    'status',
    'description',
    'secret_key',
    'currency_id',
    'merchant_id',
    'is_h2h',
    'registered_delay',
    'created_at',
]


@pytest.mark.asyncio
async def test_generate_category_schema(postgres_sessionmaker, language_context):
    category = sqlalchemy.SQLAlchemyAdmin(
        search_fields=['id'],
        model=Terminal,
        db_async_session=postgres_sessionmaker,
        table_schema=sqlalchemy.SQLAlchemyFieldsSchema(
            model=Terminal,
            fields=FIELDS,
            created_at=schema.DateTimeField(range=True),
        ),
    )
    new_schema = category.generate_schema(UserABC(username="test"), language_context)
    assert new_schema.model_dump() == category_schema_data, new_schema.model_dump()



import functools
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field, validate_call
from pydantic.dataclasses import dataclass

from brilliance_admin.schema.table.fields_schema import FieldsSchema
from brilliance_admin.translations import DataclassBase
from brilliance_admin.utils import SupportsStr, humanize_field_name


class ActionData(BaseModel):
    pks: List[Any] = Field(default_factory=list)
    form_data: dict = Field(default_factory=dict)

    search: str | None = None
    filters: Dict[str, Any] = Field(default_factory=dict)

    send_to_all: bool = False


@dataclass
class ActionMessage(DataclassBase):
    text: SupportsStr
    type: str = 'success'
    position: str = 'top-center'


@dataclass
class ActionResult(DataclassBase):
    message: ActionMessage | SupportsStr | None = None
    persistent_message: SupportsStr | None = None

    def __init__(
        self,
        message: ActionMessage | SupportsStr | None = None,
        persistent_message: SupportsStr | None = None,
    ):
        if isinstance(message, (str, SupportsStr)) and not isinstance(message, ActionMessage):
            self.message = ActionMessage(text=message)
        else:
            self.message = message
        self.persistent_message = persistent_message


# pylint: disable=too-many-arguments
# pylint: disable=too-many-positional-arguments
@validate_call
def admin_action(
    title: SupportsStr | None = None,
    description: Optional[SupportsStr] = None,
    confirmation_text: Optional[SupportsStr] = None,

    # https://vuetifyjs.com/en/styles/colors/#material-colors
    base_color: Optional[str] = None,

    # https://pictogrammers.com/library/mdi/
    icon: Optional[str] = None,

    # elevated, flat, tonal, outlined, text, and plain.
    variant: Optional[str] = None,

    allow_empty_selection: bool = False,
    form_schema: Optional[FieldsSchema] = None,
):
    def wrapper(func):
        func.__action__ = True

        func.action_info = {
            'title': title or humanize_field_name(func.__name__),
            'description': description,
            'confirmation_text': confirmation_text,

            'icon': icon,
            'base_color': base_color,
            'variant': variant,

            'allow_empty_selection': allow_empty_selection,
            'form_schema': form_schema,
        }

        @functools.wraps(func)
        async def wrapped(*args, **kwargs):
            return await func(*args, **kwargs)

        return wrapped

    return wrapper



import abc
import datetime
from enum import Enum
from typing import Any, ClassVar

from pydantic.dataclasses import dataclass

from brilliance_admin.exceptions import FieldError
from brilliance_admin.schema.category import FieldSchemaData
from brilliance_admin.translations import LanguageContext
from brilliance_admin.translations import TranslateText as _
from brilliance_admin.utils import DeserializeAction, SupportsStr, humanize_field_name


@dataclass
class TableField(abc.ABC, FieldSchemaData):
    _type: ClassVar[str]

    label: SupportsStr | None = None
    help_text: SupportsStr | None = None

    def generate_schema(self, user, field_slug, language_context: LanguageContext) -> FieldSchemaData:
        schema = FieldSchemaData(
            type=self._type,
            label=language_context.get_text(self.label) or humanize_field_name(field_slug),
            help_text=language_context.get_text(self.help_text),
            header=self.header,
            read_only=self.read_only,
            default=self.default,
            required=self.required,
        )

        return schema

    async def serialize(self, value, extra: dict, *args, **kwargs) -> Any:
        return value

    async def deserialize(self, value, action: DeserializeAction, extra: dict, *args, **kwargs) -> Any:
        if self.required and value is None:
            raise FieldError('Field is required', 'field_required')

        return value

    # pylint: disable=too-many-arguments
    # pylint: disable=too-many-positional-arguments
    def set_deserialized_value(self, result: dict, field_slug, deserialized_value, action, extra):
        result[field_slug] = deserialized_value


@dataclass
class IntegerField(TableField):
    _type = 'integer'

    choices: Any | None = None

    max_value: int | None = None
    min_value: int | None = None

    inputmode: str | None = None
    precision: int | None = None
    scale: int | None = None

    def generate_schema(self, user, field_slug, language_context: LanguageContext) -> FieldSchemaData:
        schema = super().generate_schema(user, field_slug, language_context)

        if self.max_value is not None:
            schema.max_value = self.max_value

        if self.min_value is not None:
            schema.min_value = self.min_value

        schema.inputmode = self.inputmode
        schema.precision = self.precision
        schema.scale = self.scale

        return schema

    async def deserialize(self, value, action: DeserializeAction, extra: dict, *args, **kwargs) -> Any:
        value = await super().deserialize(value, action, extra, *args, **kwargs)
        if value and not isinstance(value, int):
            raise FieldError(_('validation.bad_type_error') % {'type': type(value).__name__, 'expected': 'init'})

        if value is None:
            if self.min_value is not None:
                raise FieldError(_('validation.min_value_error') % {'min': self.min_value})
            return

        if self.min_value is not None and value < self.min_value:
            raise FieldError(_('validation.min_value_error') % {'min': self.min_value})

        if self.max_value is not None and value > self.max_value:
            raise FieldError(_('validation.max_value_error') % {'max': self.max_value})

        return value


@dataclass
class StringField(TableField):
    _type = 'string'

    multilined: bool | None = None
    ckeditor: bool | None = None
    tinymce: bool | None = None

    min_length: int | None = None
    max_length: int | None = None
    password: bool | None = False

    choices: Any | None = None

    def generate_schema(self, user, field_slug, language_context: LanguageContext) -> FieldSchemaData:
        schema = super().generate_schema(user, field_slug, language_context)

        schema.multilined = self.multilined
        schema.ckeditor = self.ckeditor
        schema.tinymce = self.tinymce
        schema.password = self.password

        if self.min_length is not None:
            schema.min_length = self.min_length

        if self.max_length is not None:
            schema.max_length = self.max_length

        return schema

    async def deserialize(self, value, action: DeserializeAction, extra: dict, *args, **kwargs) -> Any:
        value = await super().deserialize(value, action, extra, *args, **kwargs)
        if value and not isinstance(value, str):
            raise FieldError(_('validation.bad_type_error') % {'type': type(value).__name__, 'expected': 'string'})

        if value is None:
            if self.min_length is not None and self.min_length > 0:
                raise FieldError(_('validation.min_length_error') % {'min': self.min_length})
            return

        if self.min_length is not None and len(value) < self.min_length:
            raise FieldError(_('validation.min_length_error') % {'min': self.min_length})

        if self.max_length is not None and len(value) > self.max_length:
            raise FieldError(_('validation.max_length_error') % {'max': self.max_length})

        return value


@dataclass
class BooleanField(TableField):
    _type = 'boolean'


def _parse_iso(value: str) -> datetime.datetime:
    if value.endswith('Z'):
        value = value.replace('Z', '+00:00')

    dt = datetime.datetime.fromisoformat(value)

    if dt.tzinfo is None:
        dt = dt.replace(tzinfo=datetime.timezone.utc)

    return dt


@dataclass
class DateTimeField(TableField):
    _type = 'datetime'

    format: str = '%Y-%m-%dT%H:%M:%S'
    range: bool | None = None
    include_date: bool | None = True
    include_time: bool | None = True

    def generate_schema(self, user, field_slug, language_context: LanguageContext) -> FieldSchemaData:
        schema = super().generate_schema(user, field_slug, language_context)

        schema.range = self.range
        schema.include_date = self.include_date
        schema.include_time = self.include_time

        return schema

    async def deserialize(self, value, action: DeserializeAction, extra: dict, *args, **kwargs) -> Any:
        value = await super().deserialize(value, action, extra, *args, **kwargs)

        if not value:
            return

        if value and not isinstance(value, (str, dict)):
            raise FieldError(_('validation.bad_type_error') % {'type': type(value).__name__, 'expected': 'datetime'})

        if isinstance(value, str):
            return _parse_iso(value)

        if isinstance(value, dict):
            if not value.get('from') or not value.get('to'):
                raise FieldError(
                    f'{type(self).__name__} value must be dict with from,to values: {value}'
                )

            return {
                'from': _parse_iso(value['from']),
                'to': _parse_iso(value['to']),
            }

        raise FieldError(_('validation.bad_type_error') % {'type': type(value).__name__, 'expected': 'datetime'})


@dataclass
class JSONField(TableField):
    _type = 'json'

    async def deserialize(self, value, action: DeserializeAction, extra: dict, *args, **kwargs) -> Any:
        value = await super().deserialize(value, action, extra, *args, **kwargs)

        if value is None:
            return

        if not isinstance(value, (dict, list)):
            raise FieldError(_('validation.bad_type_error') % {'type': type(value).__name__, 'expected': 'JSON'})

        return value


@dataclass
class ArrayField(TableField):
    _type = 'array'

    array_type: str | None

    def generate_schema(self, user, field_slug, language_context: LanguageContext) -> FieldSchemaData:
        schema = super().generate_schema(user, field_slug, language_context)

        schema.array_type = self.array_type

        return schema

    async def deserialize(self, value, action: DeserializeAction, extra: dict, *args, **kwargs) -> Any:
        value = await super().deserialize(value, action, extra, *args, **kwargs)

        if value is None:
            return

        if not isinstance(value, list):
            raise FieldError(_('validation.bad_type_error') % {'type': type(value).__name__, 'expected': 'Array'})

        return value


@dataclass
class FileField(TableField):
    _type = 'file'


@dataclass
class ImageField(TableField):
    _type = 'image'

    preview_max_height: int = 100
    preview_max_width: int = 100

    def generate_schema(self, user, field_slug, language_context: LanguageContext) -> FieldSchemaData:
        schema = super().generate_schema(user, field_slug, language_context)

        if self.preview_max_height is not None:
            schema.preview_max_height = self.preview_max_height

        if self.preview_max_width is not None:
            schema.preview_max_width = self.preview_max_width

        return schema

    async def serialize(self, value, extra: dict, *args, **kwargs) -> Any:
        return {'url': value}


@dataclass
class ChoiceField(TableField):
    _type = 'choice'

    # Tag color available:
    # https://vuetifyjs.com/en/styles/colors/#classes
    choices: Any | None = None

    # https://vuetifyjs.com/en/components/chips/#color-and-variants
    variant: str = 'elevated'
    size: str = 'default'

    def __post_init__(self):
        self.choices = self.generate_choices()

    def generate_choices(self):
        if not self.choices:
            return None

        if issubclass(self.choices, Enum):
            return [
                {'value': c.value, 'title': c.label, 'tag_color': getattr(c, 'tag_color', None)}
                for c in self.choices
            ]

        msg = f'Field choices is not suppored: {self.choices}'
        raise NotImplementedError(msg)

    def find_choice(self, value):
        if not self.choices:
            return None

        return next((c for c in self.choices if c.get('value') == value), None)

    def generate_schema(self, user, field_slug, language_context: LanguageContext) -> FieldSchemaData:
        schema = super().generate_schema(user, field_slug, language_context)

        schema.choices = self.choices

        schema.size = self.size
        schema.variant = self.variant

        return schema

    async def serialize(self, value, extra: dict, *args, **kwargs) -> Any:
        if not value:
            return

        choice = self.find_choice(value)
        return {
            'value': value,
            'title': choice.get('title') or value if choice else value.capitalize(),
        }

    async def deserialize(self, value, action: DeserializeAction, extra: dict, *args, **kwargs) -> Any:
        value = await super().deserialize(value, action, extra, *args, **kwargs)

        if value is None:
            return

        if isinstance(value, dict):
            if 'value' not in value:
                raise FieldError(
                    f'{type(self).__name__} dict value must contain "value": {value}'
                )
            value = value['value']

        if not isinstance(value, str):
            raise FieldError(
                f'{type(self).__name__} value must be str, got {type(value)}'
            )

        choice = self.find_choice(value)
        if not choice:
            raise FieldError(
                f'Invalid choice value "{value}", allowed: {[c["value"] for c in self.choices or []]}'
            )

        return value


@dataclass
class RelatedField(TableField):
    _type = 'related'

    many: bool = False
    dual_list: bool = False
    filter_fn: Any | None = None

    def generate_schema(self, user, field_slug, language_context: LanguageContext) -> FieldSchemaData:
        schema = super().generate_schema(user, field_slug, language_context)
        schema.many = self.many
        schema.dual_list = self.dual_list
        return schema

    async def autocomplete(self, data, user, extra):
        raise NotImplementedError('autocomplete is not implemented')

    async def deserialize(self, action: DeserializeAction, extra: dict, *args, **kwargs) -> Any:
        value = await super().deserialize(action, extra, *args, **kwargs)
        if not value:
            return None

        if isinstance(value, list):
            result = []
            for i in value:
                i = i.get('key')
                if not isinstance(i, (int, str)):
                    raise FieldError(f'Value "{i}" is not supported for related field')
                result.append(i)
            return result

        result = None
        if isinstance(value, dict) and 'key' in value:
            result = value['key']
        if isinstance(value, (int, str)):
            result = value
        if not isinstance(result, (int, str)):
            raise FieldError(f'Value "{result}" is not supported for related field')
        return result



# pylint: disable=wildcard-import, unused-wildcard-import, unused-import
# flake8: noqa: F405
from .related_field import SQLAlchemyRelatedField
from .auth import SQLAlchemyJWTAdminAuthentication
from .fields_schema import SQLAlchemyFieldsSchema
from .table import *



import importlib

import pytest
from fastapi import Request

from brilliance_admin.api.views.settings import AdminSettingsData
from brilliance_admin.translations import TranslateText
from example.main import admin_schema, app

SCOPE = {
    'type': 'http',
    'method': 'GET',
    'path': '/',
    'raw_path': b'/',
    'headers': [],
    'query_string': b'',
    'scheme': 'http',
    'server': ('testserver', 80),
    'client': ('testclient', 50000),
    'root_path': '',
    'app': app,
    'asgi': {'version': '3.0'},
}


@pytest.mark.asyncio
async def test_index_context_data():
    request = Request(scope=SCOPE)
    # admin_schema.backend_prefix = 'test'
    admin_schema.custom_themes = []
    result = await admin_schema.get_index_context_data(request)
    version = importlib.metadata.version('brilliance-admin')
    assert result == {
        'favicon_image': '/static/favicon.jpg',
        'settings_json': '{"backend_prefix": "http://testserver/admin/", "static_prefix": '
        f'"http://testserver/admin/static/", "version": "{version}", "api_timeout_ms": '
        '5000, "logo_image": "http://testserver/static/logo-outline.png", "default_theme": '
        '"deepPurpleDark", "custom_themes": []}',
        'title': 'Brilliance Admin Demo',
    }


@pytest.mark.asyncio
async def test_settings():
    request = Request(scope=SCOPE)
    settings = await admin_schema.get_settings(request)
    s = AdminSettingsData(
        main_page='/main/dashboard/',
        title=TranslateText(slug='admin_title'),
        description=TranslateText(slug='admin_description'),
        login_greetings_message=TranslateText(slug='login_greetings_message'),
        navbar_density='default',
        languages={'ru': 'Russian', 'en': 'English', 'test': 'Test'},
    )
    assert settings == s, settings




def extract_integrity_detail(exc) -> str:
    orig = getattr(exc, 'orig', None)
    if orig is not None:
        orig_cause = getattr(orig, '__cause__', None)
        if orig_cause is not None:
            table = getattr(orig_cause, 'table_name', None)
            column = getattr(orig_cause, 'column_name', None)
            if table and column:
                return f'{table}.{column}'
            if table:
                return table
            msg = getattr(orig_cause, 'message', '')
            if msg:
                return msg
            if orig_cause.args:
                return str(orig_cause.args[0])
        return str(orig).split('\n')[0]
    return str(exc)



import datetime
from typing import Any

from brilliance_admin import schema
from brilliance_admin.exceptions import AdminAPIException, APIError
from brilliance_admin.integrations.sqlalchemy.related_field import SQLAlchemyRelatedField
from brilliance_admin.schema.table.fields.base import DateTimeField
from brilliance_admin.translations import TranslateText as _
from brilliance_admin.utils import DeserializeAction, humanize_field_name

FIELD_FILTERS_NOT_FOUND = '{class_name} filter "{field_slug}" not found inside table_filters fields: {available_filters}'


class SQLAlchemyFieldsSchema(schema.FieldsSchema):
    model: Any

    def __init__(self, *args, model=None, **kwargs):
        if model:
            self.model = model

        super().__init__(*args, **kwargs)

    def generate_fields(self, kwargs) -> dict:
        generated_fields = super().generate_fields(kwargs)

        # pylint: disable=import-outside-toplevel
        from sqlalchemy import inspect
        from sqlalchemy.dialects.postgresql import ARRAY
        from sqlalchemy.ext.mutable import Mutable
        from sqlalchemy.sql import sqltypes
        from sqlalchemy.sql.schema import Column

        mapper = inspect(self.model).mapper

        for attr in mapper.column_attrs:
            col: Column = attr.columns[0]
            field_slug = attr.key

            if field_slug in generated_fields:
                continue

            field_data = {}
            info = col.info or {}
            field_data["label"] = info.get('label', humanize_field_name(field_slug))
            field_data["help_text"] = info.get('help_text')

            field_data["read_only"] = col.primary_key

            # Whether the field is required on input (best-effort heuristic)
            field_data["required"] = (
                not col.nullable
                and col.default is None
                and col.server_default is None
                and not col.primary_key
            )

            col_type = col.type
            try:
                py_t = col_type.python_type
            except Exception:
                py_t = None

            impl = getattr(attr, 'impl', None)
            is_impl_mutable = isinstance(impl, Mutable)

            # Foreign key column
            if col.foreign_keys:
                continue

            elif "choices" in info:
                field_data["choices"] = info['choices']
                field_class = schema.ChoiceField

            elif isinstance(col_type, (sqltypes.BigInteger, sqltypes.Integer)) or py_t is int:
                field_class = schema.IntegerField

                if isinstance(col_type, sqltypes.SmallInteger):
                    field_data['min_value'] = -32768
                    field_data['max_value'] = 32767

                elif isinstance(col_type, sqltypes.Integer):
                    field_data['min_value'] = -2147483648
                    field_data['max_value'] = 2147483647

                elif isinstance(col_type, sqltypes.BigInteger):
                    field_data['min_value'] = -9223372036854775808
                    field_data['max_value'] = 9223372036854775807

            elif isinstance(col_type, sqltypes.Numeric):
                field_class = schema.IntegerField
                field_data["inputmode"] = "decimal"
                field_data["precision"] = col_type.precision
                field_data["scale"] = col_type.scale

            elif isinstance(col_type, sqltypes.String) or py_t is str:
                field_class = schema.StringField
                # Max length is usually stored as String(length=...)
                if getattr(col_type, "length", None):
                    field_data["max_length"] = col_type.length

            elif isinstance(col_type, sqltypes.DateTime) or py_t is datetime:
                field_class = schema.DateTimeField

            elif isinstance(col_type, sqltypes.Boolean) or py_t is bool:
                field_class = schema.BooleanField

            elif isinstance(col_type, sqltypes.JSON):
                field_class = schema.JSONField

            elif isinstance(col_type, ARRAY):
                field_class = schema.ArrayField
                field_data["array_type"] = type(col_type.item_type).__name__.lower()
                field_data["read_only"] = is_impl_mutable or isinstance(col_type, Mutable)

            elif isinstance(col_type, sqltypes.NullType):
                continue

            elif not self.fields:
                msg = f'SQLAlchemy autogenerate ORM field {self.model.__name__}.{field_slug} is not supported for type: {col_type}'
                raise AttributeError(msg)

            schema_field = field_class(**field_data)

            if col.primary_key:
                generated_fields = {field_slug: schema_field, **generated_fields}
            else:
                generated_fields[field_slug] = schema_field

        for field_slug, field in self.generate_related_fields():
            generated_fields[field_slug] = field

        return generated_fields

    def generate_related_fields(self):
        # pylint: disable=import-outside-toplevel
        from sqlalchemy import inspect

        mapper = inspect(self.model).mapper

        # relationship-поля
        for rel in mapper.relationships:
            # relationship, у которых есть локальные FK-колонки, не добавляем в схему,
            # так как связь редактируется через scalar-поле (FK),
            # а relationship используется только для ORM-навигации
            if any(col.foreign_keys for col in rel.local_columns):
                continue

            field_slug = rel.key

            field_data = {}

            info = rel.info or {}
            field_data["label"] = info.get('label', humanize_field_name(field_slug))
            field_data["help_text"] = info.get('help_text')

            field_data["read_only"] = rel.viewonly
            field_data["required"] = (
                not rel.uselist
                and all(not col.nullable for col in rel.local_columns)
            )

            field_data["rel_name"] = rel.key
            field_data["many"] = rel.uselist
            field_data["dual_list"] = rel.uselist
            field_data["target_model"] = rel.mapper.class_

            yield field_slug, SQLAlchemyRelatedField(**field_data)

        # FK-поля
        for attr in mapper.column_attrs:
            col = attr.columns[0]

            if not col.foreign_keys:
                continue

            rel_obj = None
            for rel in mapper.relationships:
                if col in rel.local_columns:
                    rel_obj = rel
                    break

            if not rel_obj:
                continue

            field_slug = attr.key

            field_data = {}

            info = col.info or {}
            field_data["label"] = info.get('label', humanize_field_name(rel_obj.key))
            field_data["help_text"] = info.get('help_text')

            field_data["read_only"] = False
            field_data["required"] = (
                not col.nullable
                and col.default is None
                and col.server_default is None
                and not col.primary_key
            )

            field_data["rel_name"] = rel_obj.key
            field_data["many"] = rel_obj.uselist
            field_data["target_model"] = rel_obj.mapper.class_

            yield field_slug, SQLAlchemyRelatedField(**field_data)

    async def apply_filters(self, stmt, filters: dict):
        # pylint: disable=import-outside-toplevel
        from sqlalchemy import String, cast

        for field_slug in filters.keys():
            field = self.get_field(field_slug)

            if not field:
                available_filters = list(self.get_fields().keys())
                msg = FIELD_FILTERS_NOT_FOUND.format(
                    class_name=type(self).__name__,
                    field_slug=field_slug,
                    available_filters=available_filters,
                )
                raise AttributeError(msg)

        deserialized_filters = await self.deserialize(
            filters,
            DeserializeAction.FILTERS,
            extra={'model': self.model},
        )

        for field_slug, value in deserialized_filters.items():
            field = self.get_field(field_slug)
            column = getattr(self.model, field_slug, None)

            apply_filter = getattr(field, 'apply_filter', None)
            if apply_filter and callable(apply_filter):
                stmt = await apply_filter(stmt, value, self.model, column)

            elif issubclass(type(field), DateTimeField) and field.range:
                stmt = stmt.where(column >= value['from'])
                stmt = stmt.where(column <= value['to'])

            elif isinstance(value, list):
                stmt = stmt.where(column.in_(value))

            elif isinstance(value, str):
                stmt = stmt.where(
                    cast(column, String).like(value)
                )

            else:
                stmt = stmt.where(column == value)

        return stmt

    async def serialize(self, record, extra: dict, *args, **kwargs) -> dict:

        # Convert model values to dict
        record_data = {}

        for slug, field in self.get_fields().items():
            # pylint: disable=protected-access
            if field._type == 'related':
                record_data[slug] = record
            else:
                record_data[slug] = getattr(record, slug, None)

        return await super().serialize(record_data, extra, *args, **kwargs)

    def validate_incoming_data(self, data):
        '''
        Validate that all fields keys has their schema

        for create, update
        '''
        for field_slug in data.keys():
            field = self.get_field(field_slug)
            if not field:
                available = list(self.get_fields().keys())
                msg = _('field_not_found_in_schema') % {'field_slug': field_slug, 'available': available}
                raise AdminAPIException(
                    APIError(message=msg, code='field_not_found_in_schema '),
                    status_code=400,
                )

    async def create(self, user, data, session):
        self.validate_incoming_data(data)

        record = self.model()

        deserialized_data = await self.deserialize(
            data,
            DeserializeAction.CREATE,
            extra={'model': self.model},
        )

        # сначала простые поля
        for field_slug, value in deserialized_data.items():
            field = self.get_field(field_slug)

            if isinstance(field, SQLAlchemyRelatedField):
                continue

            setattr(record, field_slug, value)

        session.add(record)

        # затем related под no_autoflush
        for field_slug, value in deserialized_data.items():
            field = self.get_field(field_slug)

            if isinstance(field, SQLAlchemyRelatedField):
                with session.no_autoflush:
                    await field.update_related(record, field_slug, value, session)

        await session.commit()
        await session.refresh(record)
        return record

    async def update(self, record, user, data, session):
        self.validate_incoming_data(data)

        deserialized_data = await self.deserialize(
            data,
            DeserializeAction.UPDATE,
            extra={'model': self.model},
        )

        for field_slug, value in deserialized_data.items():
            field = self.get_field(field_slug)

            if isinstance(field, SQLAlchemyRelatedField):
                await field.update_related(record, field_slug, value, session)
                continue

            setattr(record, field_slug, value)

        await session.commit()
        return record



from datetime import datetime

import pytest

from brilliance_admin import auth, schema, sqlalchemy
from brilliance_admin.schema import admin_schema
from example.sections.models import Currency, CurrencyFactory, MerchantFactory, Terminal, TerminalFactory


@pytest.mark.asyncio
async def test_list_filter(postgres_sessionmaker, language_context):
    category = sqlalchemy.SQLAlchemyAdmin(
        model=Terminal,
        db_async_session=postgres_sessionmaker,
        table_schema=sqlalchemy.SQLAlchemyFieldsSchema(
            model=Terminal,
            fields=['id'],
        ),
        table_filters=sqlalchemy.SQLAlchemyFieldsSchema(
            model=Terminal,
            fields=['id', 'title', 'created_at'],
            created_at=schema.DateTimeField(range=True),
        ),
    )
    user = auth.UserABC(username="test")

    merchant = await MerchantFactory(title="Test merch")
    currency = await CurrencyFactory()
    terminal_1 = await TerminalFactory(title='Test terminal', merchant=merchant, currency=currency)
    terminal_2 = await TerminalFactory(title='Test terminal second', merchant=merchant, currency=currency)
    await TerminalFactory(title='other', merchant=merchant, currency=currency)

    list_result: dict = await category.get_list(
        list_data=schema.ListData(filters={'id': terminal_1.id}),
        user=user,
        language_context=language_context,
        admin_schema=admin_schema,
    )
    assert list_result == schema.TableListResult(
        data=[{'id': terminal_1.id}], total_count=1
    ), 'поиск по id'

    list_result: dict = await category.get_list(
        list_data=schema.ListData(filters={'title': 'Test terminal second'}),
        user=user,
        language_context=language_context,
        admin_schema=admin_schema,
    )
    assert list_result == schema.TableListResult(
        data=[{'id': terminal_2.id}], total_count=1
    ), 'Полная строка'

    list_result: dict = await category.get_list(
        list_data=schema.ListData(filters={'title': 'Test%'}),
        user=user,
        language_context=language_context,
        admin_schema=admin_schema,
    )
    assert list_result == schema.TableListResult(
        data=[{'id': terminal_2.id}, {'id': terminal_1.id}], total_count=2
    ), 'Частичное вхождение'

    terminal_old = await TerminalFactory(
        title='Old terminal',
        merchant=merchant,
        currency=currency,
        created_at=datetime(2023, 6, 1, 12, 0, 0),
    )
    list_result: dict = await category.get_list(
        list_data=schema.ListData(filters={'created_at': {'from': '2022-12-04T18:55:00', 'to': '2023-12-17T18:55:00'}}),
        user=user,
        language_context=language_context,
        admin_schema=admin_schema,
    )
    assert list_result == schema.TableListResult(
        data=[{'id': terminal_old.id}], total_count=1
    ), 'Фильтр по периоду'


@pytest.mark.asyncio
async def test_list_search(postgres_sessionmaker, language_context):
    category = sqlalchemy.SQLAlchemyAdmin(
        search_fields=['title'],
        model=Terminal,
        db_async_session=postgres_sessionmaker,
        table_schema=sqlalchemy.SQLAlchemyFieldsSchema(
            model=Terminal,
            fields=['id'],
        ),
    )
    user = auth.UserABC(username="test")

    merchant = await MerchantFactory(title="Test merch")
    currency = await CurrencyFactory()
    terminal_1 = await TerminalFactory(title='Test terminal', merchant=merchant, currency=currency)
    terminal_2 = await TerminalFactory(title='Test terminal second', merchant=merchant, currency=currency)
    terminal_3 = await TerminalFactory(title='other', merchant=merchant, currency=currency)
    await TerminalFactory(title='other', merchant=merchant, currency=currency)

    list_result: dict = await category.get_list(
        list_data=schema.ListData(search='Test%'),
        user=user,
        language_context=language_context,
        admin_schema=admin_schema,
    )
    assert list_result == schema.TableListResult(
        data=[{'id': terminal_2.id}, {'id': terminal_1.id}], total_count=2,
    ), 'Поиск по title'


@pytest.mark.asyncio
async def test_filter_related_one(postgres_sessionmaker, language_context):
    category = sqlalchemy.SQLAlchemyAdmin(
        model=Currency,
        db_async_session=postgres_sessionmaker,
        table_schema=sqlalchemy.SQLAlchemyFieldsSchema(
            model=Currency,
            fields=['id'],
        ),
        table_filters=sqlalchemy.SQLAlchemyFieldsSchema(
            model=Currency,
            fields=['terminals'],
        ),
    )
    user = auth.UserABC(username="test")

    currency_rub = await CurrencyFactory(title='RUB')

    merchant = await MerchantFactory()
    terminal_1 = await TerminalFactory(merchant=merchant, currency=currency_rub)
    terminal_2 = await TerminalFactory(merchant=merchant, currency=currency_rub)

    currency_usd = await CurrencyFactory(title='USD')
    terminal_3 = await TerminalFactory(merchant=merchant, currency=currency_usd)

    list_result: dict = await category.get_list(
        list_data=schema.ListData(filters={
            'terminals': [{'key': terminal_1.id, 'title': 'test'}, {'key': terminal_2.id, 'title': 'test'}],
        }),
        user=user,
        language_context=language_context,
        admin_schema=admin_schema,
    )
    assert list_result == schema.TableListResult(
        data=[{'id': currency_rub.id}], total_count=1
    ), 'Фильтр по many related'


@pytest.mark.asyncio
async def test_filter_related_many(postgres_sessionmaker, language_context):
    category = sqlalchemy.SQLAlchemyAdmin(
        model=Terminal,
        db_async_session=postgres_sessionmaker,
        table_schema=sqlalchemy.SQLAlchemyFieldsSchema(
            model=Terminal,
            fields=['id'],
        ),
        table_filters=sqlalchemy.SQLAlchemyFieldsSchema(
            model=Terminal,
            fields=['merchant_id'],
        ),
    )
    user = auth.UserABC(username="test")

    currency = await CurrencyFactory()

    merchant_1 = await MerchantFactory()
    terminal_1 = await TerminalFactory(merchant=merchant_1, currency=currency)

    merchant_2 = await MerchantFactory()
    terminal_2 = await TerminalFactory(merchant=merchant_2, currency=currency)

    list_result: dict = await category.get_list(
        list_data=schema.ListData(filters={
            'merchant_id': {'key': merchant_2.id, 'title': 'test'}
        }),
        user=user,
        language_context=language_context,
        admin_schema=admin_schema,
    )
    assert list_result == schema.TableListResult(data=[{'id': terminal_2.id}], total_count=1), 'Фильтр по related'


@pytest.mark.asyncio
async def test_list_bad_search_field(postgres_sessionmaker):
    with pytest.raises(AttributeError) as e:
        sqlalchemy.SQLAlchemyAdmin(
            search_fields=['no_field'],
            model=Terminal,
            db_async_session=postgres_sessionmaker,
        )

    assert str(e.value) == 'SQLAlchemyAdmin: search field "no_field" not found in model Terminal'


@pytest.mark.asyncio
async def test_ordering(postgres_sessionmaker, language_context):
    category = sqlalchemy.SQLAlchemyAdmin(
        model=Terminal,
        db_async_session=postgres_sessionmaker,
        ordering_fields=['id'],
        table_schema=sqlalchemy.SQLAlchemyFieldsSchema(
            model=Terminal,
            fields=['id'],
        ),
    )
    user = auth.UserABC(username="test")

    currency = await CurrencyFactory()
    merchant = await MerchantFactory()

    terminal_1 = await TerminalFactory(merchant=merchant, currency=currency)
    terminal_2 = await TerminalFactory(merchant=merchant, currency=currency)

    list_result: dict = await category.get_list(
        list_data=schema.ListData(ordering='id'),
        user=user,
        language_context=language_context,
        admin_schema=admin_schema,
    )
    assert list_result == schema.TableListResult(
        data=[{'id': terminal_1.id}, {'id': terminal_2.id, }], total_count=2
    ), 'сортировка по возрастанию'

    list_result: dict = await category.get_list(
        list_data=schema.ListData(ordering='-id'),
        user=user,
        language_context=language_context,
        admin_schema=admin_schema,
    )
    assert list_result == schema.TableListResult(
        data=[{'id': terminal_2.id}, {'id': terminal_1.id, }], total_count=2
    ), 'сортировка по убыванию'



from fastapi import APIRouter, Request
from fastapi.responses import JSONResponse

from brilliance_admin.auth import AdminAuthentication, AuthData, AuthResult
from brilliance_admin.exceptions import AdminAPIException, APIError
from brilliance_admin.schema.admin_schema import AdminSchema
from brilliance_admin.translations import LanguageContext

router = APIRouter(prefix="/auth", tags=["Auth"])


@router.post(
    path='/login/',
    responses={
        401: {"model": APIError},
        400: {"model": APIError},
    },
)
async def login(request: Request, auth_data: AuthData) -> AuthResult:
    schema: AdminSchema = request.app.state.schema

    language_slug = request.headers.get('Accept-Language')
    language_context: LanguageContext = schema.get_language_context(language_slug)
    context = {'language_context': language_context}

    auth: AdminAuthentication = schema.auth
    try:
        result: AuthResult = await auth.login(auth_data, debug=schema.debug)
    except AdminAPIException as e:
        return JSONResponse(e.get_error().model_dump(mode='json', context=context), status_code=e.status_code)

    return JSONResponse(content=result.model_dump(mode='json', context=context))



import inspect

from brilliance_admin.auth import AdminAuthentication, AuthData, AuthResult, UserABC, UserResult
from brilliance_admin.exceptions import AdminAPIException, APIError
from brilliance_admin.translations import TranslateText as _
from brilliance_admin.utils import get_logger

logger = get_logger()


class SQLAlchemyJWTAdminAuthentication(AdminAuthentication):
    secret: str
    db_async_session = None
    user_model = None
    pk_name = None

    password_validator = None

    def __init__(self, secret: str, db_async_session, user_model, pk_name='id', password_validator=None):
        self.pk_name = pk_name
        self.secret = secret
        self.db_async_session = db_async_session
        self.user_model = user_model
        self.password_validator = password_validator

        if self.password_validator:
            if not callable(self.password_validator):
                raise ValueError("password_validator must be callable")

        if not isinstance(secret, str) or not secret:
            raise ValueError("JWT secret must be a non-empty string")

        # pylint: disable=import-outside-toplevel
        from sqlalchemy.inspection import inspect
        try:
            import jwt
        except ImportError as e:
            msg = "PyJWT is not installed. Install it with: pip install pyjwt"
            raise RuntimeError(msg) from e

        assert hasattr(jwt, "encode"), "PyJWT is not installed"

        mapper = inspect(user_model)
        columns = {col.key for col in mapper.columns}

        required = {self.pk_name, "username", "is_admin"}
        missing = required - columns

        if missing:
            msg = f"user_model is missing required columns: {', '.join(sorted(missing))}"
            raise ValueError(msg)

    async def login(self, data: AuthData, debug: bool = False) -> AuthResult:
        # pylint: disable=import-outside-toplevel
        from sqlalchemy import select

        stmt = select(self.user_model).where(self.user_model.username == data.username)
        try:
            async with self.db_async_session() as session:
                result = await session.execute(stmt)

        except ConnectionRefusedError as e:
            logger.exception(
                'SQLAlchemy %s login db error: %s', type(self).__name__, e,
            )
            msg = _('errors.connection_refused_error') % {'error': str(e)}
            raise AdminAPIException(
                APIError(message=msg, code='connection_refused_error'),
                status_code=500,
            ) from e

        user = result.scalar_one_or_none()

        if not user:
            raise AdminAPIException(APIError(code="user_not_found"), status_code=401)

        try:
            if self.password_validator:
                if inspect.iscoroutinefunction(self.password_validator):
                    valid_password = await self.password_validator(user, data.password)
                else:
                    valid_password = self.password_validator(user, data.password)

                if not valid_password:
                    raise AdminAPIException(APIError(code="user_not_found"), status_code=401)

        except AdminAPIException as e:
            raise e
        except Exception as e:
            msg = _('errors.password_exception') % {
                'error_type': str(e) if debug else type(e).__name__,
            }
            raise AdminAPIException(APIError(message=msg, code="password_exception"), status_code=500) from e

        if not user.is_admin:
            raise AdminAPIException(APIError(code="not_an_admin"), status_code=401)

        return AuthResult(
            token=self.get_token(user),
            user=UserResult(username=user.username),
        )

    def get_token(self, user):
        # pylint: disable=import-outside-toplevel
        import jwt

        return jwt.encode(
            {"user_pk": str(user.id)},
            self.secret,
            algorithm="HS256",
        )

    async def authenticate(self, headers: dict) -> UserABC:
        # pylint: disable=import-outside-toplevel
        import jwt
        from sqlalchemy import inspect, select

        token = headers.get("Authorization")
        if not token:
            raise AdminAPIException(
                APIError(message="Token is not presented"),
                status_code=401,
            )

        token = token.replace("Token ", "")

        try:
            payload = jwt.decode(token, self.secret, algorithms=["HS256"])
        except jwt.exceptions.DecodeError as e:
            raise AdminAPIException(
                APIError(message="Token decoding error", code="token_error"),
                status_code=401,
            ) from e

        user_pk = payload.get("user_pk")
        if not user_pk:
            raise AdminAPIException(
                APIError(message="Invalid token payload", code="token_error"),
                status_code=401,
            )

        col = inspect(self.user_model).mapper.columns[self.pk_name]
        python_type = col.type.python_type

        stmt = select(self.user_model).where(
            getattr(self.user_model, self.pk_name) == python_type(user_pk),
            self.user_model.is_admin.is_(True),
        )
        try:
            async with self.db_async_session() as session:
                result = await session.execute(stmt)
                user = result.scalar_one_or_none()

        except ConnectionRefusedError as e:
            logger.exception(
                'SQLAlchemy %s authenticate db error: %s', type(self).__name__, e,
            )
            msg = _('errors.connection_refused_error') % {'error': str(e)}
            raise AdminAPIException(
                APIError(message=msg, code='connection_refused_error'),
                status_code=500,
            ) from e

        if not user:
            raise AdminAPIException(
                APIError(message="User not found", code="user_not_found"),
                status_code=401,
            )

        return user



import pytest
from fastapi.testclient import TestClient

from brilliance_admin.schema.table.table_models import AutocompleteData
from example.main import app
from example.sections.models import UserFactory

client = TestClient(app)


@pytest.mark.asyncio
async def test_autocomplete_filter_fn(mocker):
    user_1 = await UserFactory(username='active', is_active=True)
    user_2 = await UserFactory(username='not_active', is_active=False)

    url = app.url_path_for(
        'autocomplete',
        group='users',
        category='usersession',
    )
    request_data = AutocompleteData(
        search_string="",
        field_slug="user_id",
        is_filter=True,
        form_data={},
        existed_choices=[],
        limit=30,
    )
    response = client.post(url, json=request_data.model_dump(mode='json'))
    assert response.status_code == 200, response.content.decode()
    response_data = {
        'results': [
            {
                'key': user_1.id,
                'title': 'User #1 "active"',
            },
        ],
    }
    assert response.json() == response_data



from typing import Any, Dict, List

from pydantic import BaseModel, Field, field_serializer

from brilliance_admin.schema.category import BaseCategory, DashboardInfoSchemaData
from brilliance_admin.schema.table.fields_schema import FieldsSchema
from brilliance_admin.translations import LanguageContext
from brilliance_admin.utils import SupportsStr


class DashboardData(BaseModel):
    search: str | None = None
    filters: Dict[str, Any] = Field(default_factory=dict)


class ChartData(BaseModel):
    data: dict
    options: dict
    width: int | None = None
    height: int = 50
    type: str = 'line'

    component_type: str = 'chart'


class Subcard(BaseModel):
    title: SupportsStr
    value: SupportsStr
    color: str | None = None


class PeriodGraph(BaseModel):
    title: SupportsStr
    value: SupportsStr
    change: int | float | None = None
    subcards: List[Subcard] = Field(default_factory=list)
    values: List[List[int | float]] = Field(default_factory=list)
    vertical: List[SupportsStr] = Field(default_factory=list)
    horizontal: List[SupportsStr] = Field(default_factory=list)
    component_type: str = 'period_graph'

    @field_serializer('horizontal', 'vertical')
    def serialize_str_list(self, val: list) -> list:
        return [str(v) for v in val]


class SmallGraph(BaseModel):
    title: SupportsStr
    value: SupportsStr
    change: int | float | None = None
    points: Dict[SupportsStr, float | int] = Field(default_factory=list)
    component_type: str = 'small_graph'


class DashboardContainer(BaseModel):
    cols: int | None = None
    md: int | None = None
    lg: int | None = None
    sm: int | None = None

    component_type: str = 'container'
    components: List[Any] = Field(default_factory=list)


class CategoryDashboard(BaseCategory):
    _type_slug: str = 'dashboard'

    search_enabled: bool = False
    search_help: SupportsStr | None = None

    table_filters: FieldsSchema | None = None

    def generate_schema(self, user, language_context: LanguageContext) -> DashboardInfoSchemaData:
        schema = super().generate_schema(user, language_context)
        dashboard_info = DashboardInfoSchemaData(
            search_enabled=self.search_enabled,
            search_help=language_context.get_text(self.search_help),
        )

        if self.table_filters:
            dashboard_info.table_filters = self.table_filters.generate_schema(user, language_context)

        schema.dashboard_info = dashboard_info
        return schema

    async def get_data(self, data: DashboardData, user) -> DashboardContainer:
        raise NotImplementedError('get_data is not implemented')



from brilliance_admin.exceptions import AdminAPIException, APIError
from brilliance_admin.integrations.sqlalchemy.utils import extract_integrity_detail
from brilliance_admin.schema.table.admin_action import ActionData, ActionResult, admin_action
from brilliance_admin.translations import TranslateText as _


class SQLAlchemyDeleteAction:
    has_delete: bool = True

    def get_actions(self):
        actions = super().get_actions()
        if not self.has_delete:
            actions.pop('delete', None)
        return actions

    @admin_action(
        title=_('delete'),
        confirmation_text=_('delete_confirmation_text'),
        base_color='red-lighten-2',
        variant='outlined',
    )
    async def delete(self, *args, action_data: ActionData, **kwargs):
        if not self.has_delete:
            raise AdminAPIException(APIError(message=_('errors.method_not_allowed')), status_code=500)

        assert action_data.pks

        # pylint: disable=import-outside-toplevel
        from sqlalchemy import inspect, select
        from sqlalchemy.exc import IntegrityError

        col = inspect(self.model).mapper.columns[self.pk_name]
        python_type = col.type.python_type
        typed_pks = [python_type(pk) for pk in action_data.pks]

        try:
            async with self.db_async_session() as session:
                stmt = select(self.model).where(getattr(self.model, self.pk_name).in_(typed_pks))
                results = (await session.execute(stmt)).scalars().all()

                if len(results) != len(typed_pks):
                    found_ids = {str(getattr(obj, self.pk_name)) for obj in results}
                    missing = [pk for pk in typed_pks if str(pk) not in found_ids]
                    raise AdminAPIException(
                        APIError(message=_('errors.items_not_found') % {'ids': ', '.join(str(pk) for pk in missing)}),
                        status_code=404,
                    )

                for obj in results:
                    await session.delete(obj)
                await session.commit()

        except IntegrityError as e:
            await session.rollback()
            detail = extract_integrity_detail(e)
            raise AdminAPIException(
                APIError(message=_('errors.delete_integrity_error') % {'detail': detail}),
                status_code=400,
            ) from e

        return ActionResult(_('deleted_successfully'))
